/*
	Loop

        Copyright (C) 1998-1999 Vito Caputo - swivel@gnugeneration.com

        Loop is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        Loop is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*

8 bit palette fading function
for fading  a palette to a target palette at a specified rate.

for use along with the sprite library (uses the palette structures for
supporting multiple palette formats)
not.  never got around to implementing the support for multiple palette formats.
oh well ill do it some time, for now it will just um be interesting when fading
from like a targa palette to a pcvga palette (b,g,r -> r,g,b) but the same goal will
still be reached. so thats ok, its just the fading itself that will look interesting.
could actually be beneficial for interesting psychadelic palette fades.

also sprite fading functions. (fade a sprite into a graphics buffer contents)
*/

/*
post release changes made to sprite fading, added fading types:
FADE_NORM
FADE_ADD
FADE_SUB

for absolute/additive/subtractive fading.  gets rid of pits in the bump mapping
along edges of faded objects which are lower than values in current sprite by
using additive (greater than only fading)

may use subtractive to only get those edges (portions which are smaller than the
current) this can be useful if your object being faded is for the most part
lower than the current and you want to avoid any artifacts.

normal is just an absolute fade, anything that isnt the same or transparent
gets faded to the goal pixel.  this is what was done in the party / official
release.

- swivel
*/ 




#include	"sprite.h"
#include	"fade.h"



int fade_pal(unsigned char *palette, unsigned char *goal, int quantity, int speed)
{
	static int	i;
	static int	r, g, b, rg, gg, bg;
	static int	activity;

	activity = 0;

	for(i = 0; i < quantity; i++) {

		rg = goal[i * 3];
		if(palette[i * 3] != rg) {
			activity = 1;
			r = palette[i * 3];
			
			if(r < rg) {
				r += speed;
				if(r > rg)
					r = rg;
				palette[i * 3] = r;
			} else {
				if(r > rg) {
					r -= speed;
					if(r < rg)
						r = rg;
					palette[i * 3] = r;
				}
			}
		}


		gg = goal[i * 3 + 1];
		if(palette[i * 3 + 1] != gg){
			activity = 1;
			g = palette[i * 3 + 1];
			
			if(g < gg) {
				g += speed;	
				if(g > gg)
					g = gg;
				palette[i * 3 + 1] = g;
			} else {
				if(g > gg) {
					g -= speed;
					if(g < gg)
						g = gg;
					palette[i * 3 + 1] = g;
				}
			}
		}

		bg = goal[i * 3 + 2];
		if(palette[i * 3 + 2] != bg) {
			activity = 1;
			b = palette[i * 3 + 2];
			
			if(b < goal[i * 3 + 2]) {
				b += speed;	
				if(b > bg)
					b = bg;
				palette[i * 3 + 2] = b;
			} else {
				if(b > bg) {
					b -= speed;
					if(b < bg)
						b = bg;
					palette[i * 3 + 2] = b;
				}
			}
		}
	}

	return activity;
}



/* fading sprites into other sprites.  current is what is modified to meet goal
by speed amount.  keep calling until zero is returned.`
*/
int fade_sprite_in(struct SPRITE *current, struct SPRITE *goal, int X, int Y, int speed, int type)
{
	static unsigned int 		x, y, width, height;
	register unsigned int		i, j;
	static int			activity;
	static unsigned int		gdiff, cdiff;
	static unsigned char 		*cur, *dst, trans, dpixel;
	static short 			spixel; 

	i = j = activity = gdiff = cdiff = 0;


	switch(type) {
		case	FADE_NORM:

			break;

		case	FADE_ADD:

			break;

		case	FADE_SUB:

			break;

		default:
			printf("Unknown fade type\n");
			return 0;
	}


	cur = current->data;
	dst = goal->data;
	trans = goal->transparent;


	if(current->width >= goal->width) {
		cdiff = current->width - goal->width;
		width = goal->width;
	} else {
		gdiff = goal->width - current->width;
		width = current->width;
	}

	if(current->height >= goal->height) {
		height = goal->height;
	} else {
		height = current->height;
	}

	/* adjust dest pointer to position desired */
	cur += (Y * current->width) + X;

	for(y = 0; y < height; y++) {
		for(x = 0; x < width; x++, i++, j++) {

			dpixel = dst[j];

			if(dpixel != trans) {	

				spixel = cur[i];

				if(spixel <= dpixel) {
/*
				if(spixel != dpixel) {
*/
					activity = 1;

					if(spixel < dpixel) {
						spixel += speed;

						if(spixel > dpixel)
							cur[i] = dpixel;
						else 
							cur[i] = spixel;
					} else {
						spixel -= speed;

						if(spixel < dpixel) 
							cur[i] = dpixel;
						else
							cur[i] = spixel;
					}
				}
			}
		}

		i += cdiff;
		j += gdiff;
	}	

	return activity;
}



/* fading sprite out of current sprite.  requires 3 sprits, currents original
sprite (what to fade back to), current, and the sprite that was faded in.
*/
int fade_sprite_out(struct SPRITE *current, struct SPRITE *goal, struct SPRITE *map, int X, int Y, int speed, int type)
{
	static unsigned int 		x, y, width, height;
	register unsigned int		i, j, k;
	static int			activity;
	static unsigned int		cdiff, gdiff, mdiff;
	static unsigned char 		*cur, *dst, *mapdata, trans, dpixel, mpixel;
	static short 			spixel; 

	i = j = k = activity = cdiff = gdiff = mdiff = 0;


	switch(type) {
		case	FADE_NORM:

			break;

		case	FADE_ADD:

			break;

		case	FADE_SUB:

			break;

		default:
			printf("Unknown fade type\n");
			return 0;
	}


	cur = current->data;
	dst = goal->data;
	mapdata = map->data;
	trans = map->transparent;

	if(current->width >= map->width) {
		cdiff = current->width - map->width;
		gdiff = goal->width - map->width;
		width = map->width;
	} else {
		mdiff = map->width - current->width;
		width = current->width;
	}

		

	if(current->height >= map->height) {
		height = map->height;
	} else {
		height = current->height;
	}

	/* adjust dest pointer to position desired */
	cur += (Y * current->width) + X;
	dst += (Y * goal->width) + X;






	for(y = 0; y < height; y++) {
		for(x = 0; x < width; x++, i++, j++, k++) {

			mpixel = mapdata[j];

			if(mpixel != trans) {	

				dpixel = dst[k];
				spixel = cur[i];

				if(spixel != dpixel) {
					activity = 1;

					if(spixel < dpixel) {
						spixel += speed;

						if(spixel > dpixel)
							cur[i] = dpixel;
						else 
							cur[i] = spixel;
					} else {
						spixel -= speed;

						if(spixel < dpixel) 
							cur[i] = dpixel;
						else
							cur[i] = spixel;
					}
				}
			}
		}

		i += cdiff;
		j += mdiff;
		k += gdiff;
	}	

	return activity;
}



int fade_sprite_center_in(struct SPRITE *current, struct SPRITE *goal, int speed, int type)
{
	static int x, y;

	x = current->width - goal->width;
	y = current->height - goal->height;

	/* halves for center */
	x >>= 1;
	y >>= 1;

	return fade_sprite_in(current, goal, x, y, speed, type);
}



int fade_sprite_center_out(struct SPRITE *current, struct SPRITE *goal, struct SPRITE *map, int speed, int type)
{
	static int x, y;

	x = current->width - map->width;
	y = current->height - map->height;

	/* halves for center */
	x >>= 1;
	y >>= 1;

	return fade_sprite_out(current, goal, map, x, y, speed, type);
}
