/*
	Loop

        Copyright (C) 1998-1999 Vito Caputo - swivel@gnugeneration.com

        Loop is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        Loop is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	image
	file format
	5			char		"IMAGE" - ID
	1			ushort		width
	1			ushort		height
	(width * height)	uchar		data

	palette
	file format
	3			char		"PAL" - ID
	1			uchar		start
	1			ushort		quantity
	(quantity * 3)		uchar		data
			palette data will be stored in 8 bpe format (0-255)
			r,g,b order
*/


#include	<stdio.h>
#include        <sys/types.h>
#include        <sys/stat.h>
#include        <fcntl.h>
#include	"image.h"

struct im * imload_image(char *path)
{
	char		ID[5];
	struct	im	*image;
	int		file;

	file = open(path, O_RDONLY);
	if(file == -1) {
		printf("Error opening image: %s\n", path);
		return NULL;
	}

	image = imnew_image_struct();
	if(image == NULL) return NULL;
		

	read(file, ID, 5);
	if(strncmp("IMAGE", ID, 5) != 0) {
		imfree_image(image);
		return NULL;
	}

	read(file, &image->width, 2);
	read(file, &image->height, 2);

	image->data = (unsigned char *)calloc(1, image->width * image->height);
	if(image->data == NULL) {
		printf("Error allocating %i bytes for image data\n",
			image->width * image->height);
		imfree_image(image);
		return NULL;
	}

	read(file, image->data, image->width * image->height);

	close(file);

	return(image);
}



struct pal * imload_palette(char *path)
{
	char		ID[3];
	struct	pal	*palette;
	int		file;


	file = open(path, O_RDONLY);
	if(file == -1) {
		printf("Error opening palette: %s\n", path);
		return NULL;
	}

	palette = imnew_palette_struct();
	if(palette == NULL) {
		return NULL;
	}

	read(file, ID, 3);
	if(strncmp("PAL", ID, 3) != 0) {
		imfree_palette(palette);
		 return NULL;
	}

	read(file, &palette->start, 1);
	read(file, &palette->quantity, 2);
	palette->data = (unsigned char *)calloc(1, palette->quantity * 3);
	if(palette->data == NULL) {
		printf("Error allocating %i bytes for palette data\n",
			palette->quantity * 3);
		imfree_palette(palette);
		return NULL;
	}

	read(file, palette->data, palette->quantity * 3);

	close(file);

	return palette;
}


int	imsave_image(struct im *image, char *path)
{
	int	file;

	if(image == NULL) {
		printf("Invalid image data\n");
		return 0;
	}

	file = open(path, O_WRONLY | O_CREAT, 0666);
	if(file == -1) {
		printf("Error creating file: %s\n", path);
		return 0;
	}

	write(file, "IMAGE", 5);
	write(file, &image->width, 2);
	write(file, &image->height, 2);
	write(file, image->data, image->width * image->height);

	close(file);

	return 1;
}


int	imsave_palette(struct pal *palette, char *path)
{
	int	file;

	if(palette == NULL) {
		printf("Invalid palette data\n");
		return 0;
	}

	file = open(path, O_WRONLY | O_CREAT, 0666);
	if(file == -1) {
		printf("Error creating file: %s\n", path);
		return 0;
	}

	write(file, "PAL", 3);
	write(file, &palette->start, 1);
	write(file, &palette->quantity, 2);
	write(file, palette->data, palette->quantity * 3);

	close(file);

	return 1;
}


struct im * imnew_image_struct(void)
{
	struct im  *image;

	image = (struct im *)calloc(1, sizeof(struct im));
	if(image == NULL) {
		printf("Error allocating %i bytes for image structure\n");
		return NULL;
	}

	image->data = NULL;

	return image;
}


struct pal * imnew_palette_struct(void)
{
	struct pal  *palette;

	palette = (struct pal *)calloc(1, sizeof(struct pal));
	if(palette == NULL) {
		printf("Error allocating %i bytes for palette structure\n");
		return NULL;
	}

	palette->data = NULL;

	return palette;
}

void imfree_image(struct im *image)
{

	if(image->data != NULL) 
		free(image->data);

	if(image != NULL)
		free(image);

	return;
}


void imfree_palette(struct pal *palette)
{
	if(palette->data != NULL)
		free(palette->data);

	if(palette != NULL)
		free(palette);

	return;
}
